@@ -1,4 +1,4 @@
-/* $OpenBSD: apps.h,v 1.24 2020/09/09 12:47:46 inoguchi Exp $ */
+/* $OpenBSD: apps.h,v 1.32 2023/04/14 15:27:13 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -207,6 +207,7 @@ int unpack_revinfo(ASN1_TIME **prevtm, int *preason, ASN1_OBJECT **phold,
 #define DB_TYPE_REV	'R'
 #define DB_TYPE_EXP	'E'
 #define DB_TYPE_VAL	'V'
+#define DB_TYPE_SUSP	'S'
 
 typedef struct db_attr_st {
 	int unique_subject;
@@ -236,7 +237,6 @@ int parse_yesno(const char *str, int def);
 X509_NAME *parse_name(char *str, long chtype, int multirdn);
 int args_verify(char ***pargs, int *pargc, int *badarg, BIO *err,
     X509_VERIFY_PARAM **pm);
-void policies_print(BIO *out, X509_STORE_CTX *ctx);
 int bio_to_mem(unsigned char **out, int maxlen, BIO *in);
 int pkey_ctrl_string(EVP_PKEY_CTX *ctx, char *value);
 int init_gen_str(BIO *err, EVP_PKEY_CTX **pctx, const char *algname,
@@ -254,14 +254,12 @@ unsigned char *next_protos_parse(unsigned short *outlen, const char *in);
 #define FORMAT_ASN1     1
 #define FORMAT_TEXT     2
 #define FORMAT_PEM      3
-#define FORMAT_NETSCAPE 4
+
 #define FORMAT_PKCS12   5
 #define FORMAT_SMIME    6
 
-#define FORMAT_IISSGC	8	/* XXX this stupid macro helps us to avoid
-				 * adding yet another param to load_*key() */
-#define FORMAT_PEMRSA	9	/* PEM RSAPubicKey format */
-#define FORMAT_ASN1RSA	10	/* DER RSAPubicKey format */
+#define FORMAT_PEMRSA	9	/* PEM RSAPublicKey format */
+#define FORMAT_ASN1RSA	10	/* DER RSAPublicKey format */
 #define FORMAT_MSBLOB	11	/* MS Key blob format */
 #define FORMAT_PVK	12	/* MS PVK file format */
 
@@ -269,8 +267,6 @@ unsigned char *next_protos_parse(unsigned short *outlen, const char *in);
 #define EXT_COPY_ADD	1
 #define EXT_COPY_ALL	2
 
-#define NETSCAPE_CERT_HDR	"certificate"
-
 #define APP_PASS_LEN	1024
 
 #define SERIAL_RAND_BITS	64
@@ -304,6 +300,7 @@ struct option {
 		OPTION_VALUE_AND,
 		OPTION_VALUE_OR,
 		OPTION_UL_VALUE_OR,
+		OPTION_ORDER,
 	} type;
 	union {
 		char **arg;
@@ -315,9 +312,11 @@ struct option {
 		int *value;
 		unsigned long *ulvalue;
 		time_t *tvalue;
+		int *order;
 	} opt;
 	const int value;
 	const unsigned long ulvalue;
+	int *order;
 };
 
 void options_usage(const struct option *opts);
@@ -326,4 +325,6 @@ int options_parse(int argc, char **argv, const struct option *opts,
 
 void show_cipher(const OBJ_NAME *name, void *arg);
 
+int pkey_check(BIO *out, EVP_PKEY *pkey, int (check_fn)(EVP_PKEY_CTX *),
+    const char *desc);
 #endif
