@@ -1,4 +1,4 @@
-/* $OpenBSD: openssl.c,v 1.29 2019/03/17 17:46:00 tb Exp $ */
+/* $OpenBSD: openssl.c,v 1.33 2023/04/25 16:11:02 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -137,8 +137,6 @@
 #define FUNC_TYPE_MD_ALG        5
 #define FUNC_TYPE_CIPHER_ALG    6
 
-int single_execution = 0;
-
 typedef struct {
         int type;
         const char *name;
@@ -154,13 +152,15 @@ FUNCTION functions[] = {
 	{ FUNC_TYPE_GENERAL, "ca", ca_main },
 	{ FUNC_TYPE_GENERAL, "certhash", certhash_main },
 	{ FUNC_TYPE_GENERAL, "ciphers", ciphers_main },
+#ifndef OPENSSL_NO_CMS
+	{ FUNC_TYPE_GENERAL, "cms", cms_main },
+#endif
 	{ FUNC_TYPE_GENERAL, "crl2pkcs7", crl2pkcs7_main },
 	{ FUNC_TYPE_GENERAL, "crl", crl_main },
 	{ FUNC_TYPE_GENERAL, "dgst", dgst_main },
 	{ FUNC_TYPE_GENERAL, "enc", enc_main },
 	{ FUNC_TYPE_GENERAL, "errstr", errstr_main },
 	{ FUNC_TYPE_GENERAL, "genpkey", genpkey_main },
-	{ FUNC_TYPE_GENERAL, "nseq", nseq_main },
 #ifndef OPENSSL_NO_OCSP
 	{ FUNC_TYPE_GENERAL, "ocsp", ocsp_main },
 #endif
@@ -346,6 +346,7 @@ static void openssl_shutdown(void);
 
 static LHASH_OF(FUNCTION) *prog_init(void);
 static int do_cmd(LHASH_OF(FUNCTION) *prog, int argc, char *argv[]);
+static void print_help(void);
 static void list_pkey(BIO * out);
 static void list_cipher(BIO * out);
 static void list_md(BIO * out);
@@ -384,13 +385,8 @@ int
 main(int argc, char **argv)
 {
 	ARGS arg;
-#define PROG_NAME_SIZE	39
-	char pname[PROG_NAME_SIZE + 1];
-	FUNCTION f, *fp;
-	const char *prompt;
-	char buf[1024];
 	char *to_free = NULL;
-	int n, i, ret = 0;
+	int i, ret = 0;
 	char *p;
 	LHASH_OF(FUNCTION) * prog = NULL;
 	long errline;
@@ -452,74 +448,21 @@ main(int argc, char **argv)
 
 	prog = prog_init();
 
-	/* first check the program name */
-	program_name(argv[0], pname, sizeof pname);
-
-	f.name = pname;
-	fp = lh_FUNCTION_retrieve(prog, &f);
-	if (fp != NULL) {
-		argv[0] = pname;
-
-		single_execution = 1;
-		ret = fp->func(argc, argv);
-		goto end;
-	}
 	/*
 	 * ok, now check that there are not arguments, if there are, run with
-	 * them, shifting the ssleay off the front
+	 * them, shifting the executable name off the front
 	 */
-	if (argc != 1) {
-		argc--;
-		argv++;
+	argc--;
+	argv++;
 
-		single_execution = 1;
-		ret = do_cmd(prog, argc, argv);
-		if (ret < 0)
-			ret = 0;
+	if (argc < 1) {
+		print_help();
 		goto end;
 	}
-	/* ok, lets enter the old 'OpenSSL>' mode */
 
-	for (;;) {
+	ret = do_cmd(prog, argc, argv);
+	if (ret < 0)
 		ret = 0;
-		p = buf;
-		n = sizeof buf;
-		i = 0;
-		for (;;) {
-			p[0] = '\0';
-			if (i++)
-				prompt = ">";
-			else
-				prompt = "OpenSSL> ";
-			fputs(prompt, stdout);
-			fflush(stdout);
-			if (!fgets(p, n, stdin))
-				goto end;
-			if (p[0] == '\0')
-				goto end;
-			i = strlen(p);
-			if (i <= 1)
-				break;
-			if (p[i - 2] != '\\')
-				break;
-			i -= 2;
-			p += i;
-			n -= i;
-		}
-		if (!chopup_args(&arg, buf, &argc, &argv))
-			break;
-
-		ret = do_cmd(prog, argc, argv);
-		if (ret < 0) {
-			ret = 0;
-			goto end;
-		}
-		if (ret != 0)
-			BIO_printf(bio_err, "error in %s\n", argv[0]);
-		(void) BIO_flush(bio_err);
-	}
-	BIO_printf(bio_err, "bad exit\n");
-	ret = 1;
 
  end:
 	free(to_free);
@@ -553,12 +496,11 @@ static int
 do_cmd(LHASH_OF(FUNCTION) * prog, int argc, char *argv[])
 {
 	FUNCTION f, *fp;
-	int i, ret = 1, tp, nl;
+	int ret = 1;
+
+	if (argc <= 0 || argv[0] == NULL)
+		return 0;
 
-	if ((argc <= 0) || (argv[0] == NULL)) {
-		ret = 0;
-		goto end;
-	}
 	f.name = argv[0];
 	fp = lh_FUNCTION_retrieve(prog, &f);
 	if (fp == NULL) {
@@ -572,9 +514,16 @@ do_cmd(LHASH_OF(FUNCTION) * prog, int argc, char *argv[])
 			fp = &f;
 		}
 	}
-	if (fp != NULL) {
-		ret = fp->func(argc, argv);
-	} else if ((strncmp(argv[0], "no-", 3)) == 0) {
+
+	if (fp != NULL)
+		return fp->func(argc, argv);
+
+	if (strcmp(argv[0], "help") == 0) {
+		print_help();
+		return 0;
+	}
+
+	if ((strncmp(argv[0], "no-", 3)) == 0) {
 		BIO *bio_stdout = BIO_new_fp(stdout, BIO_NOCLOSE);
 		f.name = argv[0] + 3;
 		ret = (lh_FUNCTION_retrieve(prog, &f) != NULL);
@@ -583,14 +532,10 @@ do_cmd(LHASH_OF(FUNCTION) * prog, int argc, char *argv[])
 		else
 			BIO_printf(bio_stdout, "%s\n", argv[0] + 3);
 		BIO_free_all(bio_stdout);
-		goto end;
-	} else if ((strcmp(argv[0], "quit") == 0) ||
-	    (strcmp(argv[0], "q") == 0) ||
-	    (strcmp(argv[0], "exit") == 0) ||
-	    (strcmp(argv[0], "bye") == 0)) {
-		ret = -1;
-		goto end;
-	} else if ((strcmp(argv[0], LIST_STANDARD_COMMANDS) == 0) ||
+		return ret;
+	}
+
+	if ((strcmp(argv[0], LIST_STANDARD_COMMANDS) == 0) ||
 	    (strcmp(argv[0], LIST_MESSAGE_DIGEST_COMMANDS) == 0) ||
 	    (strcmp(argv[0], LIST_MESSAGE_DIGEST_ALGORITHMS) == 0) ||
 	    (strcmp(argv[0], LIST_CIPHER_COMMANDS) == 0) ||
@@ -626,50 +571,58 @@ do_cmd(LHASH_OF(FUNCTION) * prog, int argc, char *argv[])
 					    fp->name);
 		}
 		BIO_free_all(bio_stdout);
-		ret = 0;
-		goto end;
-	} else {
-		BIO_printf(bio_err,
-		    "openssl:Error: '%s' is an invalid command.\n",
-		    argv[0]);
-		BIO_printf(bio_err, "\nStandard commands");
-		i = 0;
-		tp = 0;
-		for (fp = functions; fp->name != NULL; fp++) {
-			nl = 0;
-#ifdef OPENSSL_NO_CAMELLIA
-			if (((i++) % 5) == 0)
-#else
-			if (((i++) % 4) == 0)
-#endif
-			{
-				BIO_printf(bio_err, "\n");
-				nl = 1;
-			}
-			if (fp->type != tp) {
-				tp = fp->type;
-				if (!nl)
-					BIO_printf(bio_err, "\n");
-				if (tp == FUNC_TYPE_MD) {
-					i = 1;
-					BIO_printf(bio_err,
-					    "\nMessage Digest commands (see the `dgst' command for more details)\n");
-				} else if (tp == FUNC_TYPE_CIPHER) {
-					i = 1;
-					BIO_printf(bio_err, "\nCipher commands (see the `enc' command for more details)\n");
-				}
-			}
-#ifdef OPENSSL_NO_CAMELLIA
-			BIO_printf(bio_err, "%-15s", fp->name);
-#else
-			BIO_printf(bio_err, "%-18s", fp->name);
-#endif
-		}
-		BIO_printf(bio_err, "\n\n");
-		ret = 0;
+		return 0;
 	}
- end:
-	return (ret);
+
+	BIO_printf(bio_err,
+	    "openssl:Error: '%s' is an invalid command.\n",
+	    argv[0]);
+	print_help();
+
+	return 1;
+}
+
+static void
+print_help(void)
+{
+	FUNCTION *fp;
+	int i = 0;
+	int tp = 0;
+	int nl;
+
+	BIO_printf(bio_err, "\nStandard commands");
+	for (fp = functions; fp->name != NULL; fp++) {
+		nl = 0;
+#ifdef OPENSSL_NO_CAMELLIA
+		if (((i++) % 5) == 0)
+#else
+		if (((i++) % 4) == 0)
+#endif
+		{
+			BIO_printf(bio_err, "\n");
+			nl = 1;
+		}
+		if (fp->type != tp) {
+			tp = fp->type;
+			if (!nl)
+				BIO_printf(bio_err, "\n");
+			if (tp == FUNC_TYPE_MD) {
+				i = 1;
+				BIO_printf(bio_err,
+				    "\nMessage Digest commands (see the `dgst' command for more details)\n");
+			} else if (tp == FUNC_TYPE_CIPHER) {
+				i = 1;
+				BIO_printf(bio_err, "\nCipher commands (see the `enc' command for more details)\n");
+			}
+		}
+#ifdef OPENSSL_NO_CAMELLIA
+		BIO_printf(bio_err, "%-15s", fp->name);
+#else
+		BIO_printf(bio_err, "%-18s", fp->name);
+#endif
+	}
+
+	BIO_printf(bio_err, "\n\n");
 }
 
 static int
