@@ -1,4 +1,4 @@
-/* $OpenBSD: speed.c,v 1.30 2023/02/23 14:55:54 tb Exp $ */
+/* $OpenBSD: speed.c,v 1.33 2023/05/20 12:03:02 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -79,6 +79,8 @@
 #define ECDSA_SECONDS   10
 #define ECDH_SECONDS    10
 
+#define MAX_UNALIGN	16
+
 #include <math.h>
 #include <signal.h>
 #include <stdio.h>
@@ -175,7 +177,7 @@ void speed_alarm_free(int run);
 #define RSA_NUM		4
 #define DSA_NUM		3
 
-#define EC_NUM       16
+#define EC_NUM       6
 #define MAX_ECDH_SIZE 256
 
 static const char *names[ALGOR_NUM] = {
@@ -236,7 +238,9 @@ KDF1_SHA1(const void *in, size_t inlen, void *out, size_t * outlen)
 int
 speed_main(int argc, char **argv)
 {
+	unsigned char *real_buf = NULL, *real_buf2 = NULL;
 	unsigned char *buf = NULL, *buf2 = NULL;
+	size_t unaligned = 0;
 	int mret = 1;
 	long count = 0, save_count = 0;
 	int i, j, k;
@@ -374,16 +378,6 @@ speed_main(int argc, char **argv)
 #define R_EC_P256    3
 #define R_EC_P384    4
 #define R_EC_P521    5
-#define R_EC_K163    6
-#define R_EC_K233    7
-#define R_EC_K283    8
-#define R_EC_K409    9
-#define R_EC_K571    10
-#define R_EC_B163    11
-#define R_EC_B233    12
-#define R_EC_B283    13
-#define R_EC_B409    14
-#define R_EC_B571    15
 
 	RSA *rsa_key[RSA_NUM];
 	long rsa_c[RSA_NUM][2];
@@ -403,53 +397,24 @@ speed_main(int argc, char **argv)
 	 * name to the following arrays and increase the EC_NUM value
 	 * accordingly.
 	 */
-	static unsigned int test_curves[EC_NUM] =
-	{
-		/* Prime Curves */
+	static unsigned int test_curves[EC_NUM] = {
 		NID_secp160r1,
 		NID_X9_62_prime192v1,
 		NID_secp224r1,
 		NID_X9_62_prime256v1,
 		NID_secp384r1,
 		NID_secp521r1,
-		/* Binary Curves */
-		NID_sect163k1,
-		NID_sect233k1,
-		NID_sect283k1,
-		NID_sect409k1,
-		NID_sect571k1,
-		NID_sect163r2,
-		NID_sect233r1,
-		NID_sect283r1,
-		NID_sect409r1,
-		NID_sect571r1
 	};
-	static const char *test_curves_names[EC_NUM] =
-	{
-		/* Prime Curves */
+	static const char *test_curves_names[EC_NUM] = {
 		"secp160r1",
 		"nistp192",
 		"nistp224",
 		"nistp256",
 		"nistp384",
 		"nistp521",
-		/* Binary Curves */
-		"nistk163",
-		"nistk233",
-		"nistk283",
-		"nistk409",
-		"nistk571",
-		"nistb163",
-		"nistb233",
-		"nistb283",
-		"nistb409",
-		"nistb571"
 	};
-	static int test_curves_bits[EC_NUM] =
-	{
+	static int test_curves_bits[EC_NUM] = {
 		160, 192, 224, 256, 384, 521,
-		163, 233, 283, 409, 571,
-		163, 233, 283, 409, 571
 	};
 
 #endif
@@ -477,8 +442,8 @@ speed_main(int argc, char **argv)
 	int decrypt = 0;
 #ifndef _WIN32
 	int multi = 0;
-	const char *errstr = NULL;
 #endif
+	const char *errstr = NULL;
 
 	if (pledge("stdio proc", NULL) == -1) {
 		perror("pledge");
@@ -500,11 +465,11 @@ speed_main(int argc, char **argv)
 	for (i = 0; i < RSA_NUM; i++)
 		rsa_key[i] = NULL;
 
-	if ((buf = malloc(BUFSIZE)) == NULL) {
+	if ((buf = real_buf = malloc(BUFSIZE + MAX_UNALIGN)) == NULL) {
 		BIO_printf(bio_err, "out of memory\n");
 		goto end;
 	}
-	if ((buf2 = malloc(BUFSIZE)) == NULL) {
+	if ((buf2 = real_buf2 = malloc(BUFSIZE + MAX_UNALIGN)) == NULL) {
 		BIO_printf(bio_err, "out of memory\n");
 		goto end;
 	}
@@ -528,11 +493,11 @@ speed_main(int argc, char **argv)
 	argc--;
 	argv++;
 	while (argc) {
-		if ((argc > 0) && (strcmp(*argv, "-elapsed") == 0)) {
+		if (argc > 0 && strcmp(*argv, "-elapsed") == 0) {
 			usertime = 0;
 			j--;	/* Otherwise, -elapsed gets confused with an
 				 * algorithm. */
-		} else if ((argc > 0) && (strcmp(*argv, "-evp") == 0)) {
+		} else if (argc > 0 && strcmp(*argv, "-evp") == 0) {
 			argc--;
 			argv++;
 			if (argc == 0) {
@@ -548,13 +513,12 @@ speed_main(int argc, char **argv)
 				goto end;
 			}
 			doit[D_EVP] = 1;
-		} else if (argc > 0 && !strcmp(*argv, "-decrypt")) {
+		} else if (argc > 0 && strcmp(*argv, "-decrypt") == 0) {
 			decrypt = 1;
 			j--;	/* Otherwise, -decrypt gets confused with an
 				 * algorithm. */
-		}
 #ifndef _WIN32
-		else if ((argc > 0) && (strcmp(*argv, "-multi") == 0)) {
+		} else if (argc > 0 && strcmp(*argv, "-multi") == 0) {
 			argc--;
 			argv++;
 			if (argc == 0) {
@@ -568,9 +532,25 @@ speed_main(int argc, char **argv)
 			}
 			j--;	/* Otherwise, -multi gets confused with an
 				 * algorithm. */
-		}
 #endif
-		else if (argc > 0 && !strcmp(*argv, "-mr")) {
+		} else if (argc > 0 && strcmp(*argv, "-unaligned") == 0) {
+			argc--;
+			argv++;
+			if (argc == 0) {
+				BIO_printf(bio_err, "no alignment offset given\n");
+				goto end;
+			}
+			unaligned = strtonum(argv[0], 0, MAX_UNALIGN, &errstr);
+			if (errstr) {
+				BIO_printf(bio_err, "bad alignment offset: %s",
+				    errstr);
+				goto end;
+			}
+			buf = real_buf + unaligned;
+			buf2 = real_buf2 + unaligned;
+			j--;	/* Otherwise, -unaligned gets confused with an
+				 * algorithm. */
+		} else if (argc > 0 && strcmp(*argv, "-mr") == 0) {
 			mr = 1;
 			j--;	/* Otherwise, -mr gets confused with an
 				 * algorithm. */
@@ -766,26 +746,6 @@ speed_main(int argc, char **argv)
 			ecdsa_doit[R_EC_P384] = 2;
 		else if (strcmp(*argv, "ecdsap521") == 0)
 			ecdsa_doit[R_EC_P521] = 2;
-		else if (strcmp(*argv, "ecdsak163") == 0)
-			ecdsa_doit[R_EC_K163] = 2;
-		else if (strcmp(*argv, "ecdsak233") == 0)
-			ecdsa_doit[R_EC_K233] = 2;
-		else if (strcmp(*argv, "ecdsak283") == 0)
-			ecdsa_doit[R_EC_K283] = 2;
-		else if (strcmp(*argv, "ecdsak409") == 0)
-			ecdsa_doit[R_EC_K409] = 2;
-		else if (strcmp(*argv, "ecdsak571") == 0)
-			ecdsa_doit[R_EC_K571] = 2;
-		else if (strcmp(*argv, "ecdsab163") == 0)
-			ecdsa_doit[R_EC_B163] = 2;
-		else if (strcmp(*argv, "ecdsab233") == 0)
-			ecdsa_doit[R_EC_B233] = 2;
-		else if (strcmp(*argv, "ecdsab283") == 0)
-			ecdsa_doit[R_EC_B283] = 2;
-		else if (strcmp(*argv, "ecdsab409") == 0)
-			ecdsa_doit[R_EC_B409] = 2;
-		else if (strcmp(*argv, "ecdsab571") == 0)
-			ecdsa_doit[R_EC_B571] = 2;
 		else if (strcmp(*argv, "ecdsa") == 0) {
 			for (i = 0; i < EC_NUM; i++)
 				ecdsa_doit[i] = 1;
@@ -802,26 +762,6 @@ speed_main(int argc, char **argv)
 			ecdh_doit[R_EC_P384] = 2;
 		else if (strcmp(*argv, "ecdhp521") == 0)
 			ecdh_doit[R_EC_P521] = 2;
-		else if (strcmp(*argv, "ecdhk163") == 0)
-			ecdh_doit[R_EC_K163] = 2;
-		else if (strcmp(*argv, "ecdhk233") == 0)
-			ecdh_doit[R_EC_K233] = 2;
-		else if (strcmp(*argv, "ecdhk283") == 0)
-			ecdh_doit[R_EC_K283] = 2;
-		else if (strcmp(*argv, "ecdhk409") == 0)
-			ecdh_doit[R_EC_K409] = 2;
-		else if (strcmp(*argv, "ecdhk571") == 0)
-			ecdh_doit[R_EC_K571] = 2;
-		else if (strcmp(*argv, "ecdhb163") == 0)
-			ecdh_doit[R_EC_B163] = 2;
-		else if (strcmp(*argv, "ecdhb233") == 0)
-			ecdh_doit[R_EC_B233] = 2;
-		else if (strcmp(*argv, "ecdhb283") == 0)
-			ecdh_doit[R_EC_B283] = 2;
-		else if (strcmp(*argv, "ecdhb409") == 0)
-			ecdh_doit[R_EC_B409] = 2;
-		else if (strcmp(*argv, "ecdhb571") == 0)
-			ecdh_doit[R_EC_B571] = 2;
 		else if (strcmp(*argv, "ecdh") == 0) {
 			for (i = 0; i < EC_NUM; i++)
 				ecdh_doit[i] = 1;
@@ -894,11 +834,7 @@ speed_main(int argc, char **argv)
 
 			BIO_printf(bio_err, "dsa512   dsa1024  dsa2048\n");
 			BIO_printf(bio_err, "ecdsap160 ecdsap192 ecdsap224 ecdsap256 ecdsap384 ecdsap521\n");
-			BIO_printf(bio_err, "ecdsak163 ecdsak233 ecdsak283 ecdsak409 ecdsak571\n");
-			BIO_printf(bio_err, "ecdsab163 ecdsab233 ecdsab283 ecdsab409 ecdsab571 ecdsa\n");
 			BIO_printf(bio_err, "ecdhp160  ecdhp192  ecdhp224  ecdhp256  ecdhp384  ecdhp521\n");
-			BIO_printf(bio_err, "ecdhk163  ecdhk233  ecdhk283  ecdhk409  ecdhk571\n");
-			BIO_printf(bio_err, "ecdhb163  ecdhb233  ecdhb283  ecdhb409  ecdhb571  ecdh\n");
 
 #ifndef OPENSSL_NO_IDEA
 			BIO_printf(bio_err, "idea     ");
@@ -935,6 +871,7 @@ speed_main(int argc, char **argv)
 #ifndef _WIN32
 			BIO_printf(bio_err, "-multi n        run n benchmarks in parallel.\n");
 #endif
+			BIO_printf(bio_err, "-unaligned n    use buffers with offset n from proper alignment.\n");
 			goto end;
 		}
 		argc--;
@@ -1952,8 +1889,8 @@ show_res:
 
  end:
 	ERR_print_errors(bio_err);
-	free(buf);
-	free(buf2);
+	free(real_buf);
+	free(real_buf2);
 	for (i = 0; i < RSA_NUM; i++)
 		if (rsa_key[i] != NULL)
 			RSA_free(rsa_key[i]);
