@@ -1,4 +1,4 @@
-/* $OpenBSD: bn_gcd.c,v 1.21 2023/01/21 09:21:11 jsing Exp $ */
+/* $OpenBSD: bn_gcd.c,v 1.27 2023/04/09 18:38:59 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -119,65 +119,87 @@ euclid(BIGNUM *a, BIGNUM *b)
 	BIGNUM *t;
 	int shifts = 0;
 
-
-	/* 0 <= b <= a */
+	/* Loop invariant: 0 <= b <= a. */
 	while (!BN_is_zero(b)) {
-		/* 0 < b <= a */
+		if (BN_is_odd(a) && BN_is_odd(b)) {
+			if (!BN_sub(a, a, b))
+				goto err;
+			if (!BN_rshift1(a, a))
+				goto err;
+		} else if (BN_is_odd(a) && !BN_is_odd(b)) {
+			if (!BN_rshift1(b, b))
+				goto err;
+		} else if (!BN_is_odd(a) && BN_is_odd(b)) {
+			if (!BN_rshift1(a, a))
+				goto err;
+		} else {
+			if (!BN_rshift1(a, a))
+				goto err;
+			if (!BN_rshift1(b, b))
+				goto err;
+			shifts++;
+			continue;
+		}
 
-		if (BN_is_odd(a)) {
-			if (BN_is_odd(b)) {
-				if (!BN_sub(a, a, b))
-					goto err;
-				if (!BN_rshift1(a, a))
-					goto err;
-				if (BN_cmp(a, b) < 0) {
-					t = a;
-					a = b;
-					b = t;
-				}
-			}
-			else		/* a odd - b even */
-			{
-				if (!BN_rshift1(b, b))
-					goto err;
-				if (BN_cmp(a, b) < 0) {
-					t = a;
-					a = b;
-					b = t;
-				}
-			}
+		if (BN_cmp(a, b) < 0) {
+			t = a;
+			a = b;
+			b = t;
 		}
-		else			/* a is even */
-		{
-			if (BN_is_odd(b)) {
-				if (!BN_rshift1(a, a))
-					goto err;
-				if (BN_cmp(a, b) < 0) {
-					t = a;
-					a = b;
-					b = t;
-				}
-			}
-			else		/* a even - b even */
-			{
-				if (!BN_rshift1(a, a))
-					goto err;
-				if (!BN_rshift1(b, b))
-					goto err;
-				shifts++;
-			}
-		}
-		/* 0 <= b <= a */
 	}
 
 	if (shifts) {
 		if (!BN_lshift(a, a, shifts))
 			goto err;
 	}
-	return (a);
 
-err:
-	return (NULL);
+	return a;
+
+ err:
+	return NULL;
+}
+
+int
+BN_gcd(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
+{
+	BIGNUM *a, *b, *t;
+	int ret = 0;
+
+	BN_CTX_start(ctx);
+	if ((a = BN_CTX_get(ctx)) == NULL)
+		goto err;
+	if ((b = BN_CTX_get(ctx)) == NULL)
+		goto err;
+
+	if (!bn_copy(a, in_a))
+		goto err;
+	if (!bn_copy(b, in_b))
+		goto err;
+	a->neg = 0;
+	b->neg = 0;
+
+	if (BN_cmp(a, b) < 0) {
+		t = a;
+		a = b;
+		b = t;
+	}
+	t = euclid(a, b);
+	if (t == NULL)
+		goto err;
+
+	if (!bn_copy(r, t))
+		goto err;
+	ret = 1;
+
+ err:
+	BN_CTX_end(ctx);
+	return (ret);
+}
+
+int
+BN_gcd_nonct(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
+{
+	return BN_gcd(r, in_a, in_b, ctx);
 }
 
 /*
@@ -201,7 +223,6 @@ BN_gcd_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	BN_init(&local_A);
 	BN_init(&local_B);
 
-
 	BN_CTX_start(ctx);
 	if ((A = BN_CTX_get(ctx)) == NULL)
 		goto err;
@@ -221,9 +242,9 @@ BN_gcd_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	if (!BN_one(X))
 		goto err;
 	BN_zero(Y);
-	if (BN_copy(B, a) == NULL)
+	if (!bn_copy(B, a))
 		goto err;
-	if (BN_copy(A, n) == NULL)
+	if (!bn_copy(A, n))
 		goto err;
 	A->neg = 0;
 
@@ -314,54 +335,16 @@ BN_gcd_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	 *      A == gcd(a,n);
 	 */
 
-	if (!BN_copy(R, A))
+	if (!bn_copy(R, A))
 		goto err;
 	ret = R;
-err:
+ err:
 	if ((ret == NULL) && (in == NULL))
 		BN_free(R);
 	BN_CTX_end(ctx);
 	return (ret);
 }
 
-int
-BN_gcd(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
-{
-	BIGNUM *a, *b, *t;
-	int ret = 0;
-
-
-	BN_CTX_start(ctx);
-	if ((a = BN_CTX_get(ctx)) == NULL)
-		goto err;
-	if ((b = BN_CTX_get(ctx)) == NULL)
-		goto err;
-
-	if (BN_copy(a, in_a) == NULL)
-		goto err;
-	if (BN_copy(b, in_b) == NULL)
-		goto err;
-	a->neg = 0;
-	b->neg = 0;
-
-	if (BN_cmp(a, b) < 0) {
-		t = a;
-		a = b;
-		b = t;
-	}
-	t = euclid(a, b);
-	if (t == NULL)
-		goto err;
-
-	if (BN_copy(r, t) == NULL)
-		goto err;
-	ret = 1;
-
-err:
-	BN_CTX_end(ctx);
-	return (ret);
-}
-
 int
 BN_gcd_ct(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
 {
@@ -370,12 +353,6 @@ BN_gcd_ct(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
 	return 1;
 }
 
-int
-BN_gcd_nonct(BIGNUM *r, const BIGNUM *in_a, const BIGNUM *in_b, BN_CTX *ctx)
-{
-	return BN_gcd(r, in_a, in_b, ctx);
-}
-
 /* BN_mod_inverse_no_branch is a special version of BN_mod_inverse.
  * It does not contain branches that may leak sensitive information.
  */
@@ -389,7 +366,6 @@ BN_mod_inverse_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	BIGNUM *ret = NULL;
 	int sign;
 
-
 	BN_init(&local_A);
 	BN_init(&local_B);
 
@@ -419,9 +395,9 @@ BN_mod_inverse_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	if (!BN_one(X))
 		goto err;
 	BN_zero(Y);
-	if (BN_copy(B, a) == NULL)
+	if (!bn_copy(B, a))
 		goto err;
-	if (BN_copy(A, n) == NULL)
+	if (!bn_copy(A, n))
 		goto err;
 	A->neg = 0;
 
@@ -524,7 +500,7 @@ BN_mod_inverse_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	if (BN_is_one(A)) {
 		/* Y*a == 1  (mod |n|) */
 		if (!Y->neg && BN_ucmp(Y, n) < 0) {
-			if (!BN_copy(R, Y))
+			if (!bn_copy(R, Y))
 				goto err;
 		} else {
 			if (!BN_nnmod(R, Y, n, ctx))
@@ -536,7 +512,7 @@ BN_mod_inverse_no_branch(BIGNUM *in, const BIGNUM *a, const BIGNUM *n,
 	}
 	ret = R;
 
-err:
+ err:
 	if ((ret == NULL) && (in == NULL))
 		BN_free(R);
 	BN_CTX_end(ctx);
@@ -555,7 +531,6 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 	if (ct)
 		return BN_mod_inverse_no_branch(in, a, n, ctx);
 
-
 	BN_CTX_start(ctx);
 	if ((A = BN_CTX_get(ctx)) == NULL)
 		goto err;
@@ -582,9 +557,9 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 	if (!BN_one(X))
 		goto err;
 	BN_zero(Y);
-	if (BN_copy(B, a) == NULL)
+	if (!bn_copy(B, a))
 		goto err;
-	if (BN_copy(A, n) == NULL)
+	if (!bn_copy(A, n))
 		goto err;
 	A->neg = 0;
 	if (B->neg || (BN_ucmp(B, A) >= 0)) {
@@ -635,7 +610,6 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 					goto err;
 			}
 
-
 			/* Same for  A  and  Y.  Afterwards, (2) still holds. */
 			shift = 0;
 			while (!BN_is_bit_set(A, shift)) /* note that 0 < A */
@@ -655,7 +629,6 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 					goto err;
 			}
 
-
 			/* We still have (1) and (2).
 			 * Both  A  and  B  are odd.
 			 * The following computations ensure that
@@ -779,7 +752,7 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 					if (!BN_lshift(tmp, X, 2))
 						goto err;
 				} else if (D->top == 1) {
-					if (!BN_copy(tmp, X))
+					if (!bn_copy(tmp, X))
 						goto err;
 					if (!BN_mul_word(tmp, D->d[0]))
 						goto err;
@@ -815,7 +788,7 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 	if (BN_is_one(A)) {
 		/* Y*a == 1  (mod |n|) */
 		if (!Y->neg && BN_ucmp(Y, n) < 0) {
-			if (!BN_copy(R, Y))
+			if (!bn_copy(R, Y))
 				goto err;
 		} else {
 			if (!BN_nnmod(R, Y,n, ctx))
@@ -827,7 +800,7 @@ BN_mod_inverse_internal(BIGNUM *in, const BIGNUM *a, const BIGNUM *n, BN_CTX *ct
 	}
 	ret = R;
 
-err:
+ err:
 	if ((ret == NULL) && (in == NULL))
 		BN_free(R);
 	BN_CTX_end(ctx);
