@@ -1,4 +1,4 @@
-/* $OpenBSD: bn_mont.c,v 1.52 2023/03/07 09:42:09 jsing Exp $ */
+/* $OpenBSD: bn_mont.c,v 1.59 2023/04/30 05:21:20 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -138,15 +138,6 @@ BN_MONT_CTX_new(void)
 	return mctx;
 }
 
-void
-BN_MONT_CTX_init(BN_MONT_CTX *mctx)
-{
-	memset(mctx, 0, sizeof(*mctx));
-
-	BN_init(&mctx->RR);
-	BN_init(&mctx->N);
-}
-
 void
 BN_MONT_CTX_free(BN_MONT_CTX *mctx)
 {
@@ -166,9 +157,9 @@ BN_MONT_CTX_copy(BN_MONT_CTX *dst, BN_MONT_CTX *src)
 	if (dst == src)
 		return dst;
 
-	if (!BN_copy(&dst->RR, &src->RR))
+	if (!bn_copy(&dst->RR, &src->RR))
 		return NULL;
-	if (!BN_copy(&dst->N, &src->N))
+	if (!bn_copy(&dst->N, &src->N))
 		return NULL;
 
 	dst->ri = src->ri;
@@ -198,7 +189,7 @@ BN_MONT_CTX_set(BN_MONT_CTX *mont, const BIGNUM *mod, BN_CTX *ctx)
 	/* Save modulus and determine length of R. */
 	if (BN_is_zero(mod))
 		goto err;
-	if (!BN_copy(&mont->N, mod))
+	if (!bn_copy(&mont->N, mod))
 		 goto err;
 	mont->N.neg = 0;
 	mont->ri = ((BN_num_bits(mod) + BN_BITS2 - 1) / BN_BITS2) * BN_BITS2;
@@ -345,25 +336,32 @@ void
 bn_montgomery_multiply_words(BN_ULONG *rp, const BN_ULONG *ap, const BN_ULONG *bp,
     const BN_ULONG *np, BN_ULONG *tp, BN_ULONG n0, int n_len)
 {
-	BN_ULONG carry1, carry2, mask, w, x;
+	BN_ULONG a0, b, carry_a, carry_n, carry, mask, w, x;
 	int i, j;
 
-	for (i = 0; i <= n_len; i++)
+	carry_a = carry_n = carry = 0;
+
+	for (i = 0; i < n_len; i++)
 		tp[i] = 0;
 
+	a0 = ap[0];
+
 	for (i = 0; i < n_len; i++) {
+		b = bp[i];
+
 		/* Compute new t[0] * n0, as we need it inside the loop. */
-		w = (ap[0] * bp[i] + tp[0]) * n0;
-	
-		carry1 = carry2 = 0;
+		w = (a0 * b + tp[0]) * n0;
+
 		for (j = 0; j < n_len; j++) {
-			bn_mulw_addw_addw(ap[j], bp[i], tp[j], carry1, &carry1, &x);
-			bn_mulw_addw_addw(np[j], w, x, carry2, &carry2, &tp[j]);
+			bn_mulw_addw_addw(ap[j], b, tp[j], carry_a, &carry_a, &x);
+			bn_mulw_addw_addw(np[j], w, x, carry_n, &carry_n, &tp[j]);
 		}
-		bn_addw_addw(carry1, carry2, tp[n_len], &tp[n_len + 1], &tp[n_len]);
+		bn_addw_addw(carry_a, carry_n, carry, &carry, &tp[n_len]);
+		carry_a = carry_n = 0;
 
 		tp++;
 	}
+	tp[n_len] = carry;
 
 	/*
 	 * The output is now in the range of [0, 2N). Attempt to reduce once by
@@ -518,7 +516,7 @@ bn_montgomery_reduce(BIGNUM *r, BIGNUM *a, BN_MONT_CTX *mctx)
 	carry = 0;
 	n0 = mctx->n0[0];
 
-	/* Add multiples of the modulus, so that it becomes divisable by R. */
+	/* Add multiples of the modulus, so that it becomes divisible by R. */
 	for (i = 0; i < n_len; i++) {
 		v = bn_mul_add_words(&a->d[i], n->d, n_len, a->d[i] * n0);
 		bn_addw_addw(v, a->d[i + n_len], carry, &carry,
@@ -561,7 +559,7 @@ BN_from_montgomery(BIGNUM *r, const BIGNUM *a, BN_MONT_CTX *mctx, BN_CTX *ctx)
 
 	if ((tmp = BN_CTX_get(ctx)) == NULL)
 		goto err;
-	if (BN_copy(tmp, a) == NULL)
+	if (!bn_copy(tmp, a))
 		goto err;
 	if (!bn_montgomery_reduce(r, tmp, mctx))
 		goto err;
