@@ -1,4 +1,4 @@
-/* $OpenBSD: dh_ameth.c,v 1.26 2022/12/26 07:18:51 jmc Exp $ */
+/* $OpenBSD: dh_ameth.c,v 1.28 2023/04/17 05:57:17 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2006.
  */
@@ -474,6 +474,24 @@ DHparams_print(BIO *bp, const DH *x)
 	return do_dh_print(bp, x, 4, NULL, 0);
 }
 
+int
+DHparams_print_fp(FILE *fp, const DH *x)
+{
+	BIO *b;
+	int ret;
+
+	if ((b = BIO_new(BIO_s_file())) == NULL) {
+		DHerror(ERR_R_BUF_LIB);
+		return 0;
+	}
+
+	BIO_set_fp(b, fp, BIO_NOCLOSE);
+	ret = DHparams_print(b, x);
+	BIO_free(b);
+
+	return ret;
+}
+
 static int
 dh_pkey_public_check(const EVP_PKEY *pkey)
 {
