@@ -1,4 +1,4 @@
-/* $OpenBSD: ec_asn1.c,v 1.41 2023/03/08 05:45:31 jsing Exp $ */
+/* $OpenBSD: ec_asn1.c,v 1.45 2023/05/04 05:59:38 tb Exp $ */
 /*
  * Written by Nils Larsch for the OpenSSL project.
  */
@@ -89,49 +89,6 @@ EC_GROUP_get_basis_type(const EC_GROUP *group)
 		return 0;
 }
 
-#ifndef OPENSSL_NO_EC2M
-int
-EC_GROUP_get_trinomial_basis(const EC_GROUP *group, unsigned int *k)
-{
-	if (group == NULL)
-		return 0;
-
-	if (EC_METHOD_get_field_type(EC_GROUP_method_of(group)) !=
-	    NID_X9_62_characteristic_two_field
-	    || !((group->poly[0] != 0) && (group->poly[1] != 0) && (group->poly[2] == 0))) {
-		ECerror(ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
-		return 0;
-	}
-	if (k)
-		*k = group->poly[1];
-
-	return 1;
-}
-
-int
-EC_GROUP_get_pentanomial_basis(const EC_GROUP *group, unsigned int *k1,
-    unsigned int *k2, unsigned int *k3)
-{
-	if (group == NULL)
-		return 0;
-
-	if (EC_METHOD_get_field_type(EC_GROUP_method_of(group)) !=
-	    NID_X9_62_characteristic_two_field
-	    || !((group->poly[0] != 0) && (group->poly[1] != 0) && (group->poly[2] != 0) && (group->poly[3] != 0) && (group->poly[4] == 0))) {
-		ECerror(ERR_R_SHOULD_NOT_HAVE_BEEN_CALLED);
-		return 0;
-	}
-	if (k1)
-		*k1 = group->poly[3];
-	if (k2)
-		*k2 = group->poly[2];
-	if (k3)
-		*k3 = group->poly[1];
-
-	return 1;
-}
-#endif
-
 /* some structures needed for the asn1 encoding */
 typedef struct x9_62_pentanomial_st {
 	long k1;
@@ -268,7 +225,6 @@ static const ASN1_ADB_TABLE X9_62_CHARACTERISTIC_TWO_adbtbl[] = {
 			.field_name = "p.onBasis",
 			.item = &ASN1_NULL_it,
 		},
-
 	},
 	{
 		.value = NID_X9_62_tpBasis,
@@ -279,7 +235,6 @@ static const ASN1_ADB_TABLE X9_62_CHARACTERISTIC_TWO_adbtbl[] = {
 			.field_name = "p.tpBasis",
 			.item = &ASN1_INTEGER_it,
 		},
-
 	},
 	{
 		.value = NID_X9_62_ppBasis,
@@ -370,7 +325,6 @@ static const ASN1_ADB_TABLE X9_62_FIELDID_adbtbl[] = {
 			.field_name = "p.prime",
 			.item = &ASN1_INTEGER_it,
 		},
-
 	},
 	{
 		.value = NID_X9_62_characteristic_two_field,
@@ -381,7 +335,6 @@ static const ASN1_ADB_TABLE X9_62_FIELDID_adbtbl[] = {
 			.field_name = "p.char_two",
 			.item = &X9_62_CHARACTERISTIC_TWO_it,
 		},
-
 	},
 };
 
@@ -662,7 +615,7 @@ EC_PRIVATEKEY_free(EC_PRIVATEKEY *a)
 
 /* some declarations of internal function */
 
-/* ec_asn1_group2field() sets the values in a X9_62_FIELDID object */
+/* ec_asn1_group2fieldid() sets the values in a X9_62_FIELDID object */
 static int ec_asn1_group2fieldid(const EC_GROUP *, X9_62_FIELDID *);
 /* ec_asn1_group2curve() sets the values in a X9_62_CURVE object */
 static int ec_asn1_group2curve(const EC_GROUP *, X9_62_CURVE *);
@@ -719,77 +672,10 @@ ec_asn1_group2fieldid(const EC_GROUP *group, X9_62_FIELDID *field)
 			ECerror(ERR_R_ASN1_LIB);
 			goto err;
 		}
-	} else			/* nid == NID_X9_62_characteristic_two_field */
-#ifdef OPENSSL_NO_EC2M
-	{
+	} else {
 		ECerror(EC_R_GF2M_NOT_SUPPORTED);
 		goto err;
 	}
-#else
-	{
-		int field_type;
-		X9_62_CHARACTERISTIC_TWO *char_two;
-
-		field->p.char_two = X9_62_CHARACTERISTIC_TWO_new();
-		char_two = field->p.char_two;
-
-		if (char_two == NULL) {
-			ECerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-		char_two->m = (long) EC_GROUP_get_degree(group);
-
-		field_type = EC_GROUP_get_basis_type(group);
-
-		if (field_type == 0) {
-			ECerror(ERR_R_EC_LIB);
-			goto err;
-		}
-		/* set base type OID */
-		if ((char_two->type = OBJ_nid2obj(field_type)) == NULL) {
-			ECerror(ERR_R_OBJ_LIB);
-			goto err;
-		}
-		if (field_type == NID_X9_62_tpBasis) {
-			unsigned int k;
-
-			if (!EC_GROUP_get_trinomial_basis(group, &k))
-				goto err;
-
-			char_two->p.tpBasis = ASN1_INTEGER_new();
-			if (!char_two->p.tpBasis) {
-				ECerror(ERR_R_MALLOC_FAILURE);
-				goto err;
-			}
-			if (!ASN1_INTEGER_set(char_two->p.tpBasis, (long) k)) {
-				ECerror(ERR_R_ASN1_LIB);
-				goto err;
-			}
-		} else if (field_type == NID_X9_62_ppBasis) {
-			unsigned int k1, k2, k3;
-
-			if (!EC_GROUP_get_pentanomial_basis(group, &k1, &k2, &k3))
-				goto err;
-
-			char_two->p.ppBasis = X9_62_PENTANOMIAL_new();
-			if (!char_two->p.ppBasis) {
-				ECerror(ERR_R_MALLOC_FAILURE);
-				goto err;
-			}
-			/* set k? values */
-			char_two->p.ppBasis->k1 = (long) k1;
-			char_two->p.ppBasis->k2 = (long) k2;
-			char_two->p.ppBasis->k3 = (long) k3;
-		} else {	/* field_type == NID_X9_62_onBasis */
-			/* for ONB the parameters are (asn1) NULL */
-			char_two->p.onBasis = ASN1_NULL_new();
-			if (!char_two->p.onBasis) {
-				ECerror(ERR_R_MALLOC_FAILURE);
-				goto err;
-			}
-		}
-	}
-#endif
 
 	ok = 1;
 
@@ -1022,8 +908,8 @@ ec_asn1_group2pkparameters(const EC_GROUP *group, ECPKPARAMETERS *params)
 	} else {
 		/* use the ECPARAMETERS structure */
 		ret->type = 1;
-		if ((ret->value.parameters = ec_asn1_group2parameters(
-			    group, NULL)) == NULL)
+		if ((ret->value.parameters = ec_asn1_group2parameters(group,
+		    NULL)) == NULL)
 			ok = 0;
 	}
 
@@ -1067,86 +953,10 @@ ec_asn1_parameters2group(const ECPARAMETERS *params)
 	}
 	/* get the field parameters */
 	tmp = OBJ_obj2nid(params->fieldID->fieldType);
-	if (tmp == NID_X9_62_characteristic_two_field)
-#ifdef OPENSSL_NO_EC2M
-	{
+	if (tmp == NID_X9_62_characteristic_two_field) {
 		ECerror(EC_R_GF2M_NOT_SUPPORTED);
 		goto err;
-	}
-#else
-	{
-		X9_62_CHARACTERISTIC_TWO *char_two;
-
-		char_two = params->fieldID->p.char_two;
-
-		field_bits = char_two->m;
-		if (field_bits > OPENSSL_ECC_MAX_FIELD_BITS) {
-			ECerror(EC_R_FIELD_TOO_LARGE);
-			goto err;
-		}
-		if ((p = BN_new()) == NULL) {
-			ECerror(ERR_R_MALLOC_FAILURE);
-			goto err;
-		}
-		/* get the base type */
-		tmp = OBJ_obj2nid(char_two->type);
-
-		if (tmp == NID_X9_62_tpBasis) {
-			long tmp_long;
-
-			if (!char_two->p.tpBasis) {
-				ECerror(EC_R_ASN1_ERROR);
-				goto err;
-			}
-			tmp_long = ASN1_INTEGER_get(char_two->p.tpBasis);
-
-			if (!(char_two->m > tmp_long && tmp_long > 0)) {
-				ECerror(EC_R_INVALID_TRINOMIAL_BASIS);
-				goto err;
-			}
-			/* create the polynomial */
-			if (!BN_set_bit(p, (int) char_two->m))
-				goto err;
-			if (!BN_set_bit(p, (int) tmp_long))
-				goto err;
-			if (!BN_set_bit(p, 0))
-				goto err;
-		} else if (tmp == NID_X9_62_ppBasis) {
-			X9_62_PENTANOMIAL *penta;
-
-			penta = char_two->p.ppBasis;
-			if (!penta) {
-				ECerror(EC_R_ASN1_ERROR);
-				goto err;
-			}
-			if (!(char_two->m > penta->k3 && penta->k3 > penta->k2 && penta->k2 > penta->k1 && penta->k1 > 0)) {
-				ECerror(EC_R_INVALID_PENTANOMIAL_BASIS);
-				goto err;
-			}
-			/* create the polynomial */
-			if (!BN_set_bit(p, (int) char_two->m))
-				goto err;
-			if (!BN_set_bit(p, (int) penta->k1))
-				goto err;
-			if (!BN_set_bit(p, (int) penta->k2))
-				goto err;
-			if (!BN_set_bit(p, (int) penta->k3))
-				goto err;
-			if (!BN_set_bit(p, 0))
-				goto err;
-		} else if (tmp == NID_X9_62_onBasis) {
-			ECerror(EC_R_NOT_IMPLEMENTED);
-			goto err;
-		} else {	/* error */
-			ECerror(EC_R_ASN1_ERROR);
-			goto err;
-		}
-
-		/* create the EC_GROUP structure */
-		ret = EC_GROUP_new_curve_GF2m(p, a, b, NULL);
-	}
-#endif
-	else if (tmp == NID_X9_62_prime_field) {
+	} else if (tmp == NID_X9_62_prime_field) {
 		/* we have a curve over a prime field */
 		/* extract the prime number */
 		if (!params->fieldID->p.prime) {
@@ -1284,7 +1094,7 @@ ec_asn1_pkparameters2group(const ECPKPARAMETERS *params)
 /* EC_GROUP <-> DER encoding of ECPKPARAMETERS */
 
 EC_GROUP *
-d2i_ECPKParameters(EC_GROUP ** a, const unsigned char **in, long len)
+d2i_ECPKParameters(EC_GROUP **a, const unsigned char **in, long len)
 {
 	EC_GROUP *group = NULL;
 	ECPKPARAMETERS *params;
@@ -1329,7 +1139,7 @@ i2d_ECPKParameters(const EC_GROUP *a, unsigned char **out)
 /* some EC_KEY functions */
 
 EC_KEY *
-d2i_ECPrivateKey(EC_KEY ** a, const unsigned char **in, long len)
+d2i_ECPrivateKey(EC_KEY **a, const unsigned char **in, long len)
 {
 	EC_KEY *ret = NULL;
 	EC_PRIVATEKEY *priv_key = NULL;
@@ -1515,7 +1325,7 @@ i2d_ECParameters(EC_KEY *a, unsigned char **out)
 }
 
 EC_KEY *
-d2i_ECParameters(EC_KEY ** a, const unsigned char **in, long len)
+d2i_ECParameters(EC_KEY **a, const unsigned char **in, long len)
 {
 	EC_KEY *ret;
 
@@ -1544,7 +1354,7 @@ d2i_ECParameters(EC_KEY ** a, const unsigned char **in, long len)
 }
 
 EC_KEY *
-o2i_ECPublicKey(EC_KEY ** a, const unsigned char **in, long len)
+o2i_ECPublicKey(EC_KEY **a, const unsigned char **in, long len)
 {
 	EC_KEY *ret = NULL;
 
