@@ -1,4 +1,4 @@
-/* $OpenBSD: m_sha1.c,v 1.17 2014/07/11 08:44:48 jsing Exp $ */
+/* $OpenBSD: m_sha1.c,v 1.23 2023/04/16 16:42:06 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -70,20 +70,23 @@
 #include <openssl/rsa.h>
 #endif
 
+#include "evp_local.h"
+#include "sha_internal.h"
+
 static int
-init(EVP_MD_CTX *ctx)
+sha1_init(EVP_MD_CTX *ctx)
 {
 	return SHA1_Init(ctx->md_data);
 }
 
 static int
-update(EVP_MD_CTX *ctx, const void *data, size_t count)
+sha1_update(EVP_MD_CTX *ctx, const void *data, size_t count)
 {
 	return SHA1_Update(ctx->md_data, data, count);
 }
 
 static int
-final(EVP_MD_CTX *ctx, unsigned char *md)
+sha1_final(EVP_MD_CTX *ctx, unsigned char *md)
 {
 	return SHA1_Final(md, ctx->md_data);
 }
@@ -92,19 +95,12 @@ static const EVP_MD sha1_md = {
 	.type = NID_sha1,
 	.pkey_type = NID_sha1WithRSAEncryption,
 	.md_size = SHA_DIGEST_LENGTH,
-	.flags = EVP_MD_FLAG_PKEY_METHOD_SIGNATURE|EVP_MD_FLAG_DIGALGID_ABSENT,
-	.init = init,
-	.update = update,
-	.final = final,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha1_init,
+	.update = sha1_update,
+	.final = sha1_final,
 	.copy = NULL,
 	.cleanup = NULL,
-#ifndef OPENSSL_NO_RSA
-	.sign = (evp_sign_method *)RSA_sign,
-	.verify = (evp_verify_method *)RSA_verify,
-	.required_pkey_type = {
-		EVP_PKEY_RSA, EVP_PKEY_RSA2, 0, 0,
-	},
-#endif
 	.block_size = SHA_CBLOCK,
 	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA_CTX),
 };
@@ -112,56 +108,44 @@ static const EVP_MD sha1_md = {
 const EVP_MD *
 EVP_sha1(void)
 {
-	return (&sha1_md);
+	return &sha1_md;
 }
 #endif
 
 #ifndef OPENSSL_NO_SHA256
 static int
-init224(EVP_MD_CTX *ctx)
+sha224_init(EVP_MD_CTX *ctx)
 {
 	return SHA224_Init(ctx->md_data);
 }
 
 static int
-init256(EVP_MD_CTX *ctx)
-{
-	return SHA256_Init(ctx->md_data);
-}
-/*
- * Even though there're separate SHA224_[Update|Final], we call
- * SHA256 functions even in SHA224 context. This is what happens
- * there anyway, so we can spare few CPU cycles:-)
- */
-static int
-update256(EVP_MD_CTX *ctx, const void *data, size_t count)
+sha224_update(EVP_MD_CTX *ctx, const void *data, size_t count)
 {
+	/*
+	 * Even though there're separate SHA224_[Update|Final], we call
+	 * SHA256 functions even in SHA224 context. This is what happens
+	 * there anyway, so we can spare few CPU cycles:-)
+	 */
 	return SHA256_Update(ctx->md_data, data, count);
 }
 
 static int
-final256(EVP_MD_CTX *ctx, unsigned char *md)
+sha224_final(EVP_MD_CTX *ctx, unsigned char *md)
 {
-	return SHA256_Final(md, ctx->md_data);
+	return SHA224_Final(md, ctx->md_data);
 }
 
 static const EVP_MD sha224_md = {
 	.type = NID_sha224,
 	.pkey_type = NID_sha224WithRSAEncryption,
 	.md_size = SHA224_DIGEST_LENGTH,
-	.flags = EVP_MD_FLAG_PKEY_METHOD_SIGNATURE|EVP_MD_FLAG_DIGALGID_ABSENT,
-	.init = init224,
-	.update = update256,
-	.final = final256,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha224_init,
+	.update = sha224_update,
+	.final = sha224_final,
 	.copy = NULL,
 	.cleanup = NULL,
-#ifndef OPENSSL_NO_RSA
-	.sign = (evp_sign_method *)RSA_sign,
-	.verify = (evp_verify_method *)RSA_verify,
-	.required_pkey_type = {
-		EVP_PKEY_RSA, EVP_PKEY_RSA2, 0, 0,
-	},
-#endif
 	.block_size = SHA256_CBLOCK,
 	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA256_CTX),
 };
@@ -169,26 +153,37 @@ static const EVP_MD sha224_md = {
 const EVP_MD *
 EVP_sha224(void)
 {
-	return (&sha224_md);
+	return &sha224_md;
+}
+
+static int
+sha256_init(EVP_MD_CTX *ctx)
+{
+	return SHA256_Init(ctx->md_data);
+}
+
+static int
+sha256_update(EVP_MD_CTX *ctx, const void *data, size_t count)
+{
+	return SHA256_Update(ctx->md_data, data, count);
+}
+
+static int
+sha256_final(EVP_MD_CTX *ctx, unsigned char *md)
+{
+	return SHA256_Final(md, ctx->md_data);
 }
 
 static const EVP_MD sha256_md = {
 	.type = NID_sha256,
 	.pkey_type = NID_sha256WithRSAEncryption,
 	.md_size = SHA256_DIGEST_LENGTH,
-	.flags = EVP_MD_FLAG_PKEY_METHOD_SIGNATURE|EVP_MD_FLAG_DIGALGID_ABSENT,
-	.init = init256,
-	.update = update256,
-	.final = final256,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha256_init,
+	.update = sha256_update,
+	.final = sha256_final,
 	.copy = NULL,
 	.cleanup = NULL,
-#ifndef OPENSSL_NO_RSA
-	.sign = (evp_sign_method *)RSA_sign,
-	.verify = (evp_verify_method *)RSA_verify,
-	.required_pkey_type = {
-		EVP_PKEY_RSA, EVP_PKEY_RSA2, 0, 0,
-	},
-#endif
 	.block_size = SHA256_CBLOCK,
 	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA256_CTX),
 };
@@ -196,52 +191,40 @@ static const EVP_MD sha256_md = {
 const EVP_MD *
 EVP_sha256(void)
 {
-	return (&sha256_md);
+	return &sha256_md;
 }
 #endif	/* ifndef OPENSSL_NO_SHA256 */
 
 #ifndef OPENSSL_NO_SHA512
 static int
-init384(EVP_MD_CTX *ctx)
+sha384_init(EVP_MD_CTX *ctx)
 {
 	return SHA384_Init(ctx->md_data);
 }
 
 static int
-init512(EVP_MD_CTX *ctx)
-{
-	return SHA512_Init(ctx->md_data);
-}
-/* See comment in SHA224/256 section */
-static int
-update512(EVP_MD_CTX *ctx, const void *data, size_t count)
+sha384_update(EVP_MD_CTX *ctx, const void *data, size_t count)
 {
+	/* See comment in SHA224/256 section */
 	return SHA512_Update(ctx->md_data, data, count);
 }
 
 static int
-final512(EVP_MD_CTX *ctx, unsigned char *md)
+sha384_final(EVP_MD_CTX *ctx, unsigned char *md)
 {
-	return SHA512_Final(md, ctx->md_data);
+	return SHA384_Final(md, ctx->md_data);
 }
 
 static const EVP_MD sha384_md = {
 	.type = NID_sha384,
 	.pkey_type = NID_sha384WithRSAEncryption,
 	.md_size = SHA384_DIGEST_LENGTH,
-	.flags = EVP_MD_FLAG_PKEY_METHOD_SIGNATURE|EVP_MD_FLAG_DIGALGID_ABSENT,
-	.init = init384,
-	.update = update512,
-	.final = final512,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha384_init,
+	.update = sha384_update,
+	.final = sha384_final,
 	.copy = NULL,
 	.cleanup = NULL,
-#ifndef OPENSSL_NO_RSA
-	.sign = (evp_sign_method *)RSA_sign,
-	.verify = (evp_verify_method *)RSA_verify,
-	.required_pkey_type = {
-		EVP_PKEY_RSA, EVP_PKEY_RSA2, 0, 0,
-	},
-#endif
 	.block_size = SHA512_CBLOCK,
 	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA512_CTX),
 };
@@ -249,26 +232,37 @@ static const EVP_MD sha384_md = {
 const EVP_MD *
 EVP_sha384(void)
 {
-	return (&sha384_md);
+	return &sha384_md;
+}
+
+static int
+sha512_init(EVP_MD_CTX *ctx)
+{
+	return SHA512_Init(ctx->md_data);
+}
+
+static int
+sha512_update(EVP_MD_CTX *ctx, const void *data, size_t count)
+{
+	return SHA512_Update(ctx->md_data, data, count);
+}
+
+static int
+sha512_final(EVP_MD_CTX *ctx, unsigned char *md)
+{
+	return SHA512_Final(md, ctx->md_data);
 }
 
 static const EVP_MD sha512_md = {
 	.type = NID_sha512,
 	.pkey_type = NID_sha512WithRSAEncryption,
 	.md_size = SHA512_DIGEST_LENGTH,
-	.flags = EVP_MD_FLAG_PKEY_METHOD_SIGNATURE|EVP_MD_FLAG_DIGALGID_ABSENT,
-	.init = init512,
-	.update = update512,
-	.final = final512,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha512_init,
+	.update = sha512_update,
+	.final = sha512_final,
 	.copy = NULL,
 	.cleanup = NULL,
-#ifndef OPENSSL_NO_RSA
-	.sign = (evp_sign_method *)RSA_sign,
-	.verify = (evp_verify_method *)RSA_verify,
-	.required_pkey_type = {
-		EVP_PKEY_RSA, EVP_PKEY_RSA2, 0, 0,
-	},
-#endif
 	.block_size = SHA512_CBLOCK,
 	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA512_CTX),
 };
@@ -276,6 +270,82 @@ static const EVP_MD sha512_md = {
 const EVP_MD *
 EVP_sha512(void)
 {
-	return (&sha512_md);
+	return &sha512_md;
+}
+
+static int
+sha512_224_init(EVP_MD_CTX *ctx)
+{
+	return SHA512_224_Init(ctx->md_data);
+}
+
+static int
+sha512_224_update(EVP_MD_CTX *ctx, const void *data, size_t count)
+{
+	return SHA512_224_Update(ctx->md_data, data, count);
+}
+
+static int
+sha512_224_final(EVP_MD_CTX *ctx, unsigned char *md)
+{
+	return SHA512_224_Final(md, ctx->md_data);
+}
+
+static const EVP_MD sha512_224_md = {
+	.type = NID_sha512_224,
+	.pkey_type = NID_sha512_224WithRSAEncryption,
+	.md_size = SHA512_224_DIGEST_LENGTH,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha512_224_init,
+	.update = sha512_224_update,
+	.final = sha512_224_final,
+	.copy = NULL,
+	.cleanup = NULL,
+	.block_size = SHA512_CBLOCK,
+	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA512_CTX),
+};
+
+const EVP_MD *
+EVP_sha512_224(void)
+{
+	return &sha512_224_md;
+}
+
+static int
+sha512_256_init(EVP_MD_CTX *ctx)
+{
+	return SHA512_256_Init(ctx->md_data);
+}
+
+static int
+sha512_256_update(EVP_MD_CTX *ctx, const void *data, size_t count)
+{
+	return SHA512_256_Update(ctx->md_data, data, count);
+}
+
+static int
+sha512_256_final(EVP_MD_CTX *ctx, unsigned char *md)
+{
+	return SHA512_256_Final(md, ctx->md_data);
+}
+
+static const EVP_MD sha512_256_md = {
+	.type = NID_sha512_256,
+	.pkey_type = NID_sha512_256WithRSAEncryption,
+	.md_size = SHA512_256_DIGEST_LENGTH,
+	.flags = EVP_MD_FLAG_DIGALGID_ABSENT,
+	.init = sha512_256_init,
+	.update = sha512_256_update,
+	.final = sha512_256_final,
+	.copy = NULL,
+	.cleanup = NULL,
+	.block_size = SHA512_CBLOCK,
+	.ctx_size = sizeof(EVP_MD *) + sizeof(SHA512_CTX),
+};
+
+const EVP_MD *
+EVP_sha512_256(void)
+{
+	return &sha512_256_md;
 }
 #endif	/* ifndef OPENSSL_NO_SHA512 */
