@@ -1,4 +1,4 @@
-/* $OpenBSD: pem_all.c,v 1.16 2016/09/04 15:53:03 jsing Exp $ */
+/* $OpenBSD: pem_all.c,v 1.20 2023/04/25 17:51:36 tb Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -239,34 +239,6 @@ PEM_write_bio_PKCS7(BIO *bp, PKCS7 *x)
 	    x, NULL, NULL, 0, NULL, NULL);
 }
 
-int
-PEM_write_NETSCAPE_CERT_SEQUENCE(FILE *fp, NETSCAPE_CERT_SEQUENCE *x)
-{
-	return PEM_ASN1_write((i2d_of_void *)i2d_NETSCAPE_CERT_SEQUENCE, PEM_STRING_X509, fp,
-	    x, NULL, NULL, 0, NULL, NULL);
-}
-
-NETSCAPE_CERT_SEQUENCE *
-PEM_read_NETSCAPE_CERT_SEQUENCE(FILE *fp, NETSCAPE_CERT_SEQUENCE **x, pem_password_cb *cb, void *u)
-{
-	return PEM_ASN1_read((d2i_of_void *)d2i_NETSCAPE_CERT_SEQUENCE, PEM_STRING_X509, fp,
-	    (void **)x, cb, u);
-}
-
-NETSCAPE_CERT_SEQUENCE *
-PEM_read_bio_NETSCAPE_CERT_SEQUENCE(BIO *bp, NETSCAPE_CERT_SEQUENCE **x, pem_password_cb *cb, void *u)
-{
-	return PEM_ASN1_read_bio((d2i_of_void *)d2i_NETSCAPE_CERT_SEQUENCE, PEM_STRING_X509, bp,
-	    (void **)x, cb, u);
-}
-
-int
-PEM_write_bio_NETSCAPE_CERT_SEQUENCE(BIO *bp, NETSCAPE_CERT_SEQUENCE *x)
-{
-	return PEM_ASN1_write_bio((i2d_of_void *)i2d_NETSCAPE_CERT_SEQUENCE, PEM_STRING_X509, bp,
-	    x, NULL, NULL, 0, NULL, NULL);
-}
-
 #ifndef OPENSSL_NO_RSA
 
 /* We treat RSA or DSA private keys as a special case.
