@@ -1,4 +1,4 @@
-/* $OpenBSD: rsa_ameth.c,v 1.27 2022/11/26 16:08:54 tb Exp $ */
+/* $OpenBSD: rsa_ameth.c,v 1.29 2023/05/19 17:31:20 tb Exp $ */
 /* Written by Dr Stephen N Henson (steve@openssl.org) for the OpenSSL
  * project 2006.
  */
@@ -64,6 +64,7 @@
 #include <openssl/bn.h>
 #include <openssl/cms.h>
 #include <openssl/err.h>
+#include <openssl/rsa.h>
 #include <openssl/x509.h>
 
 #include "asn1_local.h"
@@ -1151,6 +1152,7 @@ const EVP_PKEY_ASN1_METHOD rsa_pss_asn1_meth = {
 
 	.pkey_size = int_rsa_size,
 	.pkey_bits = rsa_bits,
+	.pkey_security_bits = rsa_security_bits,
 
 	.sig_print = rsa_sig_print,
 
