@@ -3,12 +3,22 @@
  * are enabled, rather than not being able to tell when things are
  * enabled (or possibly not yet not implemented, or removed!).
  */
-/* #define LIBRESSL_HAS_TLS1_3 */
-/* #define LIBRESSL_HAS_DTLS1_2 */
+#define LIBRESSL_HAS_QUIC
+#define LIBRESSL_HAS_TLS1_3
+#define LIBRESSL_HAS_DTLS1_2
+
+/*
+ * Used for compatibility with compilers lacking __attribute__
+ */
+#if defined(_MSC_VER) && !defined(__clang__) && !defined(__attribute__)
+#define __attribute__(a)
+#endif
 
 #define OPENSSL_THREADS
 
 #define OPENSSL_NO_BUF_FREELISTS
+#define OPENSSL_NO_DEPRECATED
+#define OPENSSL_NO_EC2M
 #define OPENSSL_NO_GMP
 #define OPENSSL_NO_JPAKE
 #define OPENSSL_NO_KRB5
@@ -55,7 +65,6 @@
 /* #define OPENSSL_NO_DTLS1_METHOD */
 #define OPENSSL_NO_DYNAMIC_ENGINE
 /* #define OPENSSL_NO_EC */
-/* #define OPENSSL_NO_EC2M */
 #define OPENSSL_NO_EC_NISTP_64_GCC_128
 #define OPENSSL_NO_EGD
 /* #define OPENSSL_NO_ENGINE */
@@ -84,7 +93,7 @@
 /* #define OPENSSL_NO_RC4 */
 #define OPENSSL_NO_RC5
 /* #define OPENSSL_NO_RDRAND */
-#define OPENSSL_NO_RFC3779
+/* #define OPENSSL_NO_RFC3779 */
 /* #define OPENSSL_NO_RMD160 */
 /* #define OPENSSL_NO_RSA */
 /* #define OPENSSL_NO_SCRYPT */
