@@ -63,7 +63,6 @@ main(int argc, char **argv)
 	BN_CTX_free(NULL);
 	BN_GENCB_free(NULL);
 	BN_MONT_CTX_free(NULL);
-	BN_RECP_CTX_free(NULL);
 	BN_clear_free(NULL);
 	BN_free(NULL);
 	BUF_MEM_free(NULL);
@@ -124,7 +123,6 @@ main(int argc, char **argv)
 	ISSUING_DIST_POINT_free(NULL);
 	NAME_CONSTRAINTS_free(NULL);
 	NCONF_free(NULL);
-	NETSCAPE_CERT_SEQUENCE_free(NULL);
 	NETSCAPE_SPKAC_free(NULL);
 	NETSCAPE_SPKI_free(NULL);
 	NOTICEREF_free(NULL);
@@ -168,16 +166,12 @@ main(int argc, char **argv)
 	POLICYQUALINFO_free(NULL);
 	POLICY_CONSTRAINTS_free(NULL);
 	POLICY_MAPPING_free(NULL);
-	PROXY_CERT_INFO_EXTENSION_free(NULL);
-	PROXY_POLICY_free(NULL);
 	RSA_OAEP_PARAMS_free(NULL);
 	RSA_PSS_PARAMS_free(NULL);
 	RSA_free(NULL);
 	RSA_meth_free(NULL);
 	SCT_LIST_free(NULL);
 	SCT_free(NULL);
-	SXNETID_free(NULL);
-	SXNET_free(NULL);
 	TS_ACCURACY_free(NULL);
 	TS_MSG_IMPRINT_free(NULL);
 	TS_REQ_ext_free(NULL);
@@ -217,7 +211,6 @@ main(int argc, char **argv)
 	X509_VERIFY_PARAM_free(NULL);
 	X509_email_free(NULL);
 	X509_free(NULL);
-	X509_policy_tree_free(NULL);
 	lh_free(NULL);
 	sk_free(NULL);
 /*	$OpenBSD: freenull.c.tail,v 1.2 2018/07/10 20:55:57 tb Exp $	*/
